﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class FlyingEnemyController : MonoBehaviour {

	private FlyingMovementMotor movementMotor;
	public AudioClip zapSound;
	private AudioSource audioManager;

	private Transform player;
	private Vector3 spawnPos;
	private float startTime;
	private bool threatRange;
	private Vector3 direction;

	private bool moveToPlayer;

	public float damageAmount = 5f;
	private LineRenderer electricArc;
	private float rechargerTime = 1f;
	private Vector3 zapNoise = Vector3.zero;

	private PlayerHealth playerHealth;

	void Awake () {
		movementMotor = GetComponent<FlyingMovementMotor> ();
		spawnPos = movementMotor.transform.position;
		movementMotor.movementTarget = spawnPos;
		movementMotor.enabled = false;
		audioManager = GetComponent<AudioSource> ();

		player = GameObject.FindGameObjectWithTag ("Player").transform;
		playerHealth = player.gameObject.GetComponent<PlayerHealth> ();

		electricArc = GetComponentInChildren<LineRenderer> ();
	}

	void Start() {
		startTime = Time.time;
		enabled = false;
	}

	void OnEnable() {
		movementMotor.enabled = true;
		moveToPlayer = true;
		movementMotor.myBody.constraints = RigidbodyConstraints.FreezePositionY |
		RigidbodyConstraints.FreezeRotationX | RigidbodyConstraints.FreezeRotationZ;
	}

	void OnDisable() {
		movementMotor.myBody.constraints = RigidbodyConstraints.FreezeAll;
		movementMotor.enabled = false;
	}

	void Update () {
		if (moveToPlayer) {
			movementMotor.movementTarget = player.position;
			direction = player.position - movementMotor.transform.position;

			threatRange = false;
			if (direction.magnitude < 2f) {
				threatRange = true;
				movementMotor.movementTarget = Vector3.zero;
			}

			rechargerTime -= Time.deltaTime;

			if (rechargerTime < 0f && threatRange &&
				Vector3.Dot (movementMotor.transform.forward, direction) > 1f) {
				zapNoise = new Vector3 (Random.Range(-1f, 1f), 0f, Random.Range(-1f, 1f)) * 0.5f;
				if (playerHealth) {
					playerHealth.TakeDamage (damageAmount / 1f + zapNoise.magnitude);
				}

				StartCoroutine (DoElectricArc());

				rechargerTime = Random.Range (1.0f, 2.0f);
			}

		} else {
			movementMotor.movementTarget = spawnPos;
			float distance = Vector3.Distance (movementMotor.transform.position, spawnPos);
			if (distance <= 1f) {
				enabled = false;
			}
		}
	}

	IEnumerator DoElectricArc() {
		audioManager.clip = zapSound;
		audioManager.Play ();

		electricArc.enabled = true;

		zapNoise = transform.rotation * zapNoise;

		float stopTime = Time.time + 0.2f;
		while (Time.time < stopTime) {
			electricArc.SetPosition (0, electricArc.transform.position);
			electricArc.SetPosition (1, player.position + zapNoise);
			Vector2 temp = electricArc.sharedMaterial.mainTextureOffset;
			temp.x = Random.value;
			electricArc.sharedMaterial.mainTextureOffset = temp;
			yield return null;
		}
		electricArc.enabled = false;
	}

	public void PlayerNotInArea() {
		moveToPlayer = false;
	}

} //class





































