﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class FlyingMovementMotor : MovementMotor {

	public float flying_Speed = 8f;
	public float zig_Zagness = 3f;
	public float zigZag_Speed = 1.1f;
	public float orientationMultiplier = 2.5f;
	public float backtrackIntensity = 0.5f;

	private Vector3 smoothedDirection = Vector3.zero;

	[HideInInspector]
	public Rigidbody myBody;

	void Awake () {
		myBody = GetComponent<Rigidbody> ();
	}

	void FixedUpdate () {
		Vector3 dir = movementTarget - transform.position;
		Vector3 zigZag = transform.right * (Mathf.PingPong (Time.time * zigZag_Speed, 2f) - 1f) *
		                 zig_Zagness;

		dir.Normalize ();

		smoothedDirection = Vector3.Slerp (smoothedDirection, dir, Time.deltaTime * 3f);
		float orientationSpeed = 1f;

		Vector3 deltaVelocity = (smoothedDirection * flying_Speed + zigZag) - myBody.velocity;
		if (Vector3.Dot (dir, transform.forward) > 0.8f) {
			myBody.AddForce (deltaVelocity, ForceMode.Force);
		} else {
			myBody.AddForce (-deltaVelocity * backtrackIntensity, ForceMode.Force);
			orientationSpeed = orientationMultiplier;
		}

		Vector3 faceDir = smoothedDirection;
		if (faceDir == Vector3.zero) {
			myBody.angularVelocity = Vector3.zero;
		} else {
			float rotationAngle = AngleAroundAxis (transform.forward, faceDir, Vector3.up);
			myBody.angularVelocity = (Vector3.up * rotationAngle * 0.2f * orientationSpeed);
		}

	}

	float AngleAroundAxis(Vector3 dirA, Vector3 dirB, Vector3 axis) {
		dirA = dirA - Vector3.Project (dirA, axis);
		dirB = dirB - Vector3.Project (dirB, axis);

		float angle = Vector3.Angle (dirA, dirB);

		return angle * (Vector3.Dot(axis, Vector3.Cross(dirA, dirB)) < 0 ? -1 : 1);
	}

} // class








































