﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RobotAnimation : MonoBehaviour {

	public Rigidbody myBody;
	public AnimationClip idle;
	public AnimationClip walk;
	public AnimationClip turnLeft;
	public AnimationClip turnRight;

	private Animation myAnim;
	private float lastAnimTime;

	void Awake () {
		myAnim = GetComponent<Animation> ();
	}

	void OnEnable() {
		myAnim [idle.name].layer = 0;
		myAnim [idle.name].weight = 1f;
		myAnim [idle.name].enabled = true;

		myAnim [walk.name].layer = 1;
		myAnim [turnLeft.name].layer = 1;
		myAnim [turnRight.name].layer = 1;

		myAnim [walk.name].weight = 1;
		myAnim [turnLeft.name].weight = 0;
		myAnim [turnRight.name].weight = 0;

		myAnim [walk.name].enabled = true;
		myAnim [turnLeft.name].enabled = true;
		myAnim [turnRight.name].enabled = true;
	}

	void FixedUpdate () {
		float turningWeight = Mathf.Abs (myBody.angularVelocity.y) * Mathf.Deg2Rad / 100f;
		float forwardWeight = myBody.velocity.magnitude / 2.5f;
		float turningDir = Mathf.Sign (myBody.angularVelocity.y);

		myAnim [walk.name].speed = Mathf.Lerp (1.0f, myAnim[walk.name].length / 
			myAnim[turnLeft.name].length * 1.33f, turningWeight);

		myAnim [turnLeft.name].time = myAnim [walk.name].time;
		myAnim [turnRight.name].time = myAnim [walk.name].time;

		myAnim [turnLeft.name].weight = Mathf.Clamp01 (-turningWeight * turningDir);
		myAnim [turnRight.name].weight = Mathf.Clamp01 (turningWeight * turningDir);
		myAnim [walk.name].weight = Mathf.Clamp01 (forwardWeight);
	}

} //class














































