﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RobotAttackController : MonoBehaviour {

	public MovementMotor motor;
	public Transform head;

	public float targetDistance_Min = 3f;
	public float targetDistance_Max = 4f;

	private EnemyAI ai;

	private Transform player;

	private bool inRange;
	private float nextRaycastTime = 0f;
	private float lastSuccessfulRaycastTime = 0f;
	private float noticeTime = 0f;

	public MonoBehaviour[] weaponBehaviors;
	public float fireFrequency = 2f;

	private bool firing;
	private float lastFireTime = -1f;
	private int nextWeaponToFire = 0;

	public MonoBehaviour patrolBehavior;

	void Awake () {
		player = GameObject.FindGameObjectWithTag ("Player").transform;
		ai = GetComponent<EnemyAI> ();
	}

	void OnEnable() {
		inRange = false;
		nextRaycastTime = Time.time + 1f;
		lastSuccessfulRaycastTime = Time.time;
		noticeTime = Time.time;
		patrolBehavior.enabled = false;
	}

	void OnDisable() {
		 Shoot(false);
		patrolBehavior.enabled = true;
	}

	void Update () {
		Vector3 playerDirection = player.position - motor.transform.position;
		playerDirection.y = 0f;
		float playerDist = playerDirection.magnitude;
		playerDirection /= playerDist;

		motor.facingDirection = playerDirection;

		if (Time.time < noticeTime + 1.5f) {
			motor.movementDirection = Vector3.zero;
			return;
		}

		if (inRange && playerDist > targetDistance_Max) {
			inRange = false;
		} else if (!inRange && playerDist < targetDistance_Min) {
			inRange = true;
		}

		if (inRange)
			motor.movementDirection = Vector3.zero;
		else 
			motor.movementDirection = playerDirection;

		if (Time.time > nextRaycastTime) {
			nextRaycastTime = Time.time + 1f;
			if (ai.CanSeePlayer ()) {
				lastSuccessfulRaycastTime = Time.time;
				if (IsAimingAtPlayer ()) {
					Shoot (true);
				} else {
					Shoot (false);
				}
			} else {
				Shoot (false);
				if(Time.time > lastSuccessfulRaycastTime + 5f) {
					ai.SendMessage ("OnLostTrack");
				}
			}
		}

		if (firing) {
			if (Time.time > lastFireTime + 1f / fireFrequency) {
				Fire ();
			}
		}

	}

	bool IsAimingAtPlayer() {
		Vector3 playerDirection = player.position - head.position;
		playerDirection.y = 0f;
		return Vector3.Angle (head.forward, playerDirection) < 15f;
	}

	void Shoot(bool state) {
		firing = state;
	}

	void Fire () {
		if (weaponBehaviors [nextWeaponToFire]) {
			weaponBehaviors [nextWeaponToFire].SendMessage ("Fire");
			nextWeaponToFire++;
			if (nextWeaponToFire == weaponBehaviors.Length) {
				nextWeaponToFire = 0;
			}
			lastFireTime = Time.time;
		}
	}

} // class






























