﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RobotMovementMotor : MovementMotor {

	public float walkingSpeed = 2.5f;
	public float turningSpeed = 100f;
	public float aimingSpeed = 150f;

	public Transform robotHead;

	private bool facingInRightDirection;
	private Quaternion headRotation = Quaternion.identity;

	private Rigidbody myBody;

	void Awake () {
		myBody = GetComponent<Rigidbody> ();
	}

	void FixedUpdate () {
		Vector3 adjustedMovementDirection = movementDirection;

		float rotationAngle;
		if (adjustedMovementDirection != Vector3.zero)
			rotationAngle = AngleAroundAxis (transform.forward, adjustedMovementDirection, Vector3.up) * 0.3f;
		else
			rotationAngle = 0f;

		Vector3 targetAngularVelocity = Vector3.up * Mathf.Clamp (rotationAngle,
			-turningSpeed * Mathf.Deg2Rad, turningSpeed * Mathf.Deg2Rad);

		myBody.angularVelocity = Vector3.MoveTowards (myBody.angularVelocity, targetAngularVelocity,
			Time.deltaTime * turningSpeed * Mathf.Deg2Rad * 3f);

		float angle = Vector3.Angle (transform.forward, adjustedMovementDirection);

		if (facingInRightDirection && angle > 25f)
			facingInRightDirection = false;
		else if (!facingInRightDirection && angle < 5)
			facingInRightDirection = true;

		Vector3 targetVelocity;
		if (facingInRightDirection) 
			targetVelocity = transform.forward * walkingSpeed + myBody.velocity.y * Vector3.up;
		else
			targetVelocity = myBody.velocity.y * Vector3.up;

		myBody.velocity = Vector3.MoveTowards (myBody.velocity, targetVelocity,
			Time.deltaTime * walkingSpeed * 3f);
	}

	void LateUpdate() {
		if (facingDirection != Vector3.zero) {
			Quaternion targetRotation = Quaternion.LookRotation (facingDirection);
			headRotation = Quaternion.RotateTowards (headRotation, targetRotation,
				aimingSpeed * Time.deltaTime);
			robotHead.rotation = headRotation * Quaternion.Inverse (transform.rotation) * robotHead.rotation;
		}
	}

	float AngleAroundAxis(Vector3 dirA, Vector3 dirB, Vector3 axis) {
		dirA = dirA - Vector3.Project (dirA, axis);
		dirB = dirB - Vector3.Project (dirB, axis);

		float angle = Vector3.Angle (dirA, dirB);

		return angle * (Vector3.Dot(axis, Vector3.Cross(dirA, dirB)) < 0 ? -1 : 1);
	}

} // class






































