﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RobotPatrolMovement : MonoBehaviour {

	public Transform[] patrolPoints;
	public MovementMotor motor;

	public float patrolPointRadius = 1f;

	private int nextPatrolPoint = 0;
	private int patrolDirection = 1;

	void OnEnable () {
		nextPatrolPoint = GetClosestPatrolPoint ();
	}

	void Update () {
		Vector3 targetVector = patrolPoints [nextPatrolPoint].position - motor.transform.position;
		targetVector.y = 0f;

		if (targetVector.sqrMagnitude < patrolPointRadius * patrolPointRadius) {
			nextPatrolPoint += patrolDirection;

			if (nextPatrolPoint == patrolPoints.Length)
				nextPatrolPoint = 0;
		}

		if (targetVector.sqrMagnitude > 1)
			targetVector.Normalize ();

		motor.movementDirection = targetVector;
		motor.facingDirection = targetVector;

	}

	int GetClosestPatrolPoint() {
		float shortestDist = Mathf.Infinity;
		int shortestIndex = 0;

		for (int i = 0; i < patrolPoints.Length; i++) {
			float dist = (patrolPoints [i].transform.position - transform.position).magnitude;
			if (dist < shortestDist) {
				shortestDist = dist;
				shortestIndex = i;
			}
		}

		if (shortestIndex < patrolPoints.Length - 1) {
			int nextIndex = (shortestIndex + 1) % patrolPoints.Length;
			float angle = Vector3.Angle (
				patrolPoints[nextIndex].position - patrolPoints[shortestIndex].position,
				patrolPoints[shortestIndex].position - transform.position
			);
			if (angle > 120)
				shortestIndex = nextIndex;
		}
		return shortestIndex;
	}

} // class



































