﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RobotSeekMissile : MonoBehaviour {

	public float speed = 15f;
	public float lifeTime = 1.5f;
	public float damageAmount = 5f;
	public float forceAmount = 5f;
	public float radius = 0.5f;
	public float seekPrecision = 2f;
	public LayerMask ignoreLayers;
	public float noise = 0f;
	public GameObject explosionPrefab;

	private Vector3 dir;
	private float spawnTime;
	private GameObject targetObject;
	private float sideBias;

	void OnEnable () {
		dir = transform.forward;
		targetObject = GameObject.FindGameObjectWithTag ("Player");
		spawnTime = Time.time;
		sideBias = Mathf.Sin (Time.time * 5f);
	}

	void OnDisable() {
		Instantiate (explosionPrefab, transform.position, Quaternion.identity);
	}

	void Update () {
		if (Time.time > spawnTime + lifeTime) {
			Destroy (gameObject);
		}

		if (targetObject) {
			Vector3 targetPos = targetObject.transform.position;
			targetPos += transform.right * (Mathf.PingPong (Time.time, 1.0f) - 0.5f) * noise;
			Vector3 targetDir = targetPos - transform.position;
			float targetDist = targetDir.magnitude;
			targetDir /= targetDist;

			if (Time.time - spawnTime < lifeTime * 0.2f && targetDist > 3f)
				targetDir += transform.right * 0.5f * sideBias;

			dir = Vector3.Slerp (dir, targetDir, Time.deltaTime * seekPrecision);

			transform.rotation = Quaternion.LookRotation (dir);
			transform.position += (dir * speed) * Time.deltaTime;
		}

		Collider[] hits = Physics.OverlapSphere (transform.position, radius, ~ignoreLayers.value);
		bool collided = false;

		foreach (Collider hit in hits) {
			if (hit.isTrigger)
				continue;

			PlayerHealth playerHealth = hit.GetComponent<PlayerHealth> ();
			if (playerHealth) {
				playerHealth.TakeDamage (damageAmount);
			}

			Rigidbody playerBody = hit.GetComponent<Rigidbody> ();
			if (playerBody) {
				Vector3 force = transform.forward * forceAmount;
				force.y = 0f;
				playerBody.AddForce (force, ForceMode.Impulse);
			}
			collided = true;
		}

		if (collided)
			Destroy (gameObject);
	}

} // class



































