﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SpiderAnimation : MonoBehaviour {

	public FreeMovementMotor movementMotor;
	public Transform spiderTransform;
	public AnimationClip activate_Anim, forward_Anim, back_Anim, left_Anim, right_Anim;

	private AudioSource audioManager;
	private Animation myAnimation;

	private bool skidding_Sounds = true;

	void Awake () {
		audioManager = GetComponent<AudioSource> ();
		myAnimation = GetComponent<Animation> ();
	}

	void OnEnable() {
		myAnimation [activate_Anim.name].enabled = true;
		myAnimation [activate_Anim.name].weight = 1f;
		myAnimation [activate_Anim.name].time = 0f;
		myAnimation [activate_Anim.name].speed = 1f;

		myAnimation [forward_Anim.name].layer = 1;
		myAnimation [forward_Anim.name].enabled = true;
		myAnimation [forward_Anim.name].weight = 0f;

		myAnimation [back_Anim.name].layer = 1;
		myAnimation [back_Anim.name].enabled = true;
		myAnimation [back_Anim.name].weight = 0f;

		myAnimation [left_Anim.name].layer = 1;
		myAnimation [left_Anim.name].enabled = true;
		myAnimation [left_Anim.name].weight = 0f;

		myAnimation [right_Anim.name].layer = 1;
		myAnimation [right_Anim.name].enabled = true;
		myAnimation [right_Anim.name].weight = 0f;
	}

	void OnDisable() {
		myAnimation [activate_Anim.name].enabled = true;
		myAnimation [activate_Anim.name].weight = 1f;
		myAnimation [activate_Anim.name].normalizedTime = 1f;
		myAnimation [activate_Anim.name].speed = -1f;
		myAnimation.CrossFade (activate_Anim.name, 0.3f, PlayMode.StopAll);
		audioManager.Stop ();
	}

	void Update () {
		Vector3 direction = movementMotor.movementDirection;
		direction.y = 0f;

		float walkWeight = direction.magnitude;

		myAnimation [forward_Anim.name].speed = walkWeight;
		myAnimation [right_Anim.name].speed = walkWeight;
		myAnimation [left_Anim.name].speed = walkWeight;
		myAnimation [back_Anim.name].speed = walkWeight;

		float angle = Mathf.DeltaAngle (
			HorizontalAngle(movementMotor.transform.forward),
			HorizontalAngle(direction)
		);

		if (walkWeight > 0.01f) {
			float w;
			if (angle < -90) {
				w = Mathf.InverseLerp (-180, -90, angle);
				myAnimation [forward_Anim.name].weight = 0;
				myAnimation [right_Anim.name].weight = 0;
				myAnimation [back_Anim.name].weight = 1f - w;
				myAnimation [left_Anim.name].weight = 1;
			} else if (angle < 0) {
				w = Mathf.InverseLerp (-90, 0, angle);
				myAnimation [forward_Anim.name].weight = w;
				myAnimation [right_Anim.name].weight = 0;
				myAnimation [back_Anim.name].weight = 0;
				myAnimation [left_Anim.name].weight = 1f - w;
			} else if (angle < 90) {
				w = Mathf.InverseLerp (0, 90, angle);
				myAnimation [forward_Anim.name].weight = 1f - w;
				myAnimation [right_Anim.name].weight = w;
				myAnimation [back_Anim.name].weight = 0;
				myAnimation [left_Anim.name].weight = 0;
			} else {
				w = Mathf.InverseLerp (90, 180, angle);
				myAnimation [forward_Anim.name].weight = 0;
				myAnimation [right_Anim.name].weight = 1f - w;
				myAnimation [back_Anim.name].weight = w;
				myAnimation [left_Anim.name].weight = 0;
			}
		}

		if (skidding_Sounds) {
			if (walkWeight > 0.2f && !audioManager.isPlaying) {
				audioManager.Play ();
			} else if(walkWeight < 0.2f && audioManager.isPlaying) {
				audioManager.Pause ();
			}
		}
	}

	float HorizontalAngle(Vector3 direction) {
		return Mathf.Atan2 (direction.x, direction.z) * Mathf.Rad2Deg;
	}

} // class



































