﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class SpiderMovementController : MonoBehaviour {

	public FreeMovementMotor movementMotor;
	public float speed = 2f;

	public Transform spiderTransform;
	private Transform player;
	private Vector3 initialPosition;

	private AudioSource audioManager;

	private float proximity_Distace = 4f;
	private float proximity_Buildup_Time = 3f;
	private float proximity_Of_No_Return = 0.6f;
	private float proximity_Level = 0f;
	private float lastSuccessfulRaycastTime;
	private float nextRaycastTime;

	private EnemyAI spiderAI;

	public GameObject explosionEffect;
	private float explosionRadius = 1.5f;
	public LayerMask playerLayer;

	public SpiderAnimation spiderAnimation;

	public SpiderGlowPlane glowPlaneScript;
	private float lastBlinkTime;

	public Renderer proximity_Renderer;

	void Awake () {
		audioManager = GetComponent<AudioSource> ();
		player = GameObject.FindGameObjectWithTag ("Player").transform;

		spiderTransform = movementMotor.transform;
		initialPosition = spiderTransform.position;

		spiderAI = GetComponent<EnemyAI> ();
	}

	void OnEnable() {
		lastSuccessfulRaycastTime = Time.time;
		spiderAnimation.enabled = true;
	}

	void OnDisable() {
		spiderAnimation.enabled = false;
	}
		
	void Update () {
		MoveTowardsPlayer ();
	}

	void MoveTowardsPlayer() {
		float distance = Vector3.Distance (spiderTransform.position, player.position);

		if (distance < 10f) {
			Vector3 playerDirection = player.position - spiderTransform.position;
			playerDirection.y = 0f;
			float player_Distace = playerDirection.magnitude;
			playerDirection /= player_Distace;

			if (distance <= 1.5f)
				movementMotor.movementDirection = Vector3.zero;
			else
				movementMotor.movementDirection = playerDirection;

			if ((player_Distace < proximity_Distace && Time.time < lastSuccessfulRaycastTime + 1f) ||
			    proximity_Level > proximity_Of_No_Return) {
				proximity_Level += Time.deltaTime / proximity_Buildup_Time;
			} else {
				proximity_Level -= Time.deltaTime / proximity_Buildup_Time;
			}

			proximity_Level = Mathf.Clamp01 (proximity_Level);

			if (proximity_Level == 1)
				Explode ();

			if (Time.time > nextRaycastTime) {
				nextRaycastTime = Time.time + 1f;
				if (spiderAI.CanSeePlayer ()) {
					lastSuccessfulRaycastTime = Time.time;
				}
			}

			float deltaBlink = 1f / Mathf.Lerp (2, 15, proximity_Level);
			if (Time.time > lastBlinkTime + deltaBlink) {
				lastBlinkTime = Time.time;
				proximity_Renderer.material.color = Color.red;
				if (audioManager.isActiveAndEnabled) {
					audioManager.Play ();
				}
				glowPlaneScript.myRenderer.enabled = !glowPlaneScript.myRenderer.enabled; 
			}

			if (Time.time > lastBlinkTime + 0.04f) {
				proximity_Renderer.material.color = Color.white;
			}

		} else {
			Vector3 initialPositionDirection = initialPosition - spiderTransform.position;
			initialPositionDirection.y = 0f;
			float initial_Distance = initialPositionDirection.magnitude;
			initialPositionDirection /= initial_Distance;

			movementMotor.movementDirection = initialPositionDirection;

			float ourDestination = Vector3.Distance (spiderTransform.position, initialPosition);

			if (ourDestination < 1f) {
				movementMotor.movementDirection = Vector3.zero;
				gameObject.SendMessage ("OnLostTrack");
			}
		}
	}

	void Explode() {
		Collider[] playerHit = Physics.OverlapSphere (transform.position, explosionRadius, playerLayer);
		if (playerHit.Length > 0) {
			playerHit [0].gameObject.GetComponent<Rigidbody> ().AddExplosionForce (10f,
				spiderTransform.position, 5f, 0.0f, ForceMode.Impulse);

			playerHit [0].gameObject.GetComponent<PlayerHealth> ().TakeDamage (15f);
		}

		Vector3 temp = spiderTransform.position;
		temp.y += 2f;
		Instantiate (explosionEffect, temp, Quaternion.identity);
		Destroy (spiderTransform.gameObject);
	}

} // class



































