﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LaserScope : MonoBehaviour {

	public float scrollSpeed = 0.3f;
	public float pulseSpeed = 1f;
	public float noiseSize = 0.5f;
	public float maxWidth = 0.3f;
	public float minWidth = 0.2f;

	public GameObject pointer;

	private LineRenderer line_Renderer;
	private string texture_Name = "_NoiseTex";

	private float anim_Time = 0f;
	private float anim_Dir = 1f;

	private PerFrameRaycast rayCast;

	void Awake () {
		line_Renderer = GetComponent<LineRenderer> ();
		rayCast = GetComponent<PerFrameRaycast> ();

		StartCoroutine (ChooseNewAnimationTargetCoroutine ());
	}

	void Update () {
		Vector2 temp = line_Renderer.material.mainTextureOffset;
		temp.x += Time.deltaTime * anim_Dir * scrollSpeed;
		line_Renderer.material.mainTextureOffset = temp;

		line_Renderer.material.SetTextureOffset (texture_Name, new Vector2(-Time.time * anim_Dir *
			scrollSpeed, 0f));

		float anim_Factor = Mathf.PingPong (Time.time * pulseSpeed, 1f);
		anim_Factor = Mathf.Max (minWidth, anim_Factor) * maxWidth;
		line_Renderer.startWidth = anim_Factor;
		line_Renderer.endWidth = anim_Factor;

		RaycastHit hitInfo = rayCast.GetHitInfo ();
		if (hitInfo.transform) {
			line_Renderer.SetPosition (1, (hitInfo.distance * Vector3.forward));
			Vector2 matScale = line_Renderer.material.mainTextureScale;
			matScale.x = 0.1f * (hitInfo.distance);
			line_Renderer.material.mainTextureScale = matScale;
			line_Renderer.material.SetTextureScale (texture_Name, new Vector2(0.1f * hitInfo.distance *
				noiseSize, noiseSize));

			if (pointer) {
				pointer.GetComponent<Renderer> ().enabled = true;
				pointer.transform.position = hitInfo.point + (transform.position - hitInfo.point) * 0.01f;
				pointer.transform.rotation = Quaternion.LookRotation (hitInfo.normal, transform.up);
				Vector3 temp_Angle = pointer.transform.eulerAngles;
				temp_Angle.x = 90f;
				pointer.transform.eulerAngles = temp_Angle;
			}

		} else {
			if (pointer)
				pointer.GetComponent<Renderer> ().enabled = false;

			float maxDist = 200f;
			line_Renderer.SetPosition (1, (maxDist * Vector3.forward));
			Vector3 temp_Scale = line_Renderer.material.mainTextureScale;
			temp_Scale.x = 0.1f * maxDist;
			line_Renderer.material.mainTextureScale = temp_Scale;
			line_Renderer.material.SetTextureScale (texture_Name, new Vector2(0.1f * maxDist *
				noiseSize, noiseSize));
		}

	}

	IEnumerator ChooseNewAnimationTargetCoroutine() {
		while (true) {
			anim_Dir = anim_Dir * 0.9f + Random.Range (0.5f, 1.5f) * 0.1f;
			yield return null;
			minWidth = minWidth * 0.8f + Random.Range (0.1f, 1.0f) * 0.2f;
			yield return new WaitForSeconds (1.0f + Random.value * 2.0f - 1.0f);
		}
	}

} // class



































